from .case_style import (CamelCase, ConstantCase, KebabCase, PascalCase,
                         SnakeCase)


class CaseChanger(object):
    def __init__(self, case):
        to_change = {
            CamelCase: self.camel_case,
            PascalCase: self.pascal_case,
            SnakeCase: self.snake_case,
            ConstantCase: self.constant_case,
            KebabCase: self.kebab_case
        }
        try:
            self.change = to_change[case]
        except KeyError:
            raise Exception('invalid case')

    def camel_case(self, words):
        """ convert text to camel case
        """
        words = [words[0].lower()] + [word.capitalize() for word in words[1:]]
        return ''.join(words)

    def pascal_case(self, words):
        """ convert text to pascal case
        """
        words = [word.capitalize() for word in words]
        return ''.join(words)

    def snake_case(self, words):
        """ convert text to snake case
        """
        words = [word.lower() for word in words]
        return '_'.join(words)

    def constant_case(self, words):
        """ convert text to constant case
        """
        words = [word.upper() for word in words]
        return '_'.join(words)

    def kebab_case(self, words):
        """ convert text to kebab case
        """
        words = [word.lower() for word in words]
        return '-'.join(words)



import re

from .case_style import (HyphenSeparated, SpaceSeparated, UnderscoreSeparated,
                         UppercaseSeparated)


class CaseGuesser(object):
    def guess(self, text):
        # TODO: for multiple matches
        if self.is_space_separated(text):
            return SpaceSeparated
        elif self.is_underscore_separated(text):
            return UnderscoreSeparated
        elif self.is_hyphen_separated(text):
            return HyphenSeparated
        elif self.is_uppercase_separated(text):
            return UppercaseSeparated
        else:
            raise Exception('invalid case')

    def is_space_separated(self, text):
        return ' ' in text

    def is_underscore_separated(self, text):
        return '_' in text

    def is_hyphen_separated(self, text):
        return '-' in text

    def is_uppercase_separated(self, text):
        result = re.search('[A-Z]', text)
        return bool(result)



import argparse
import sys

from .case_changer import CaseChanger
from .case_guesser import CaseGuesser
from .case_style import Case
from .text_splitter import TextSplitter


def main():
    args = parse_args(sys.argv[1:])
    print(change_case_style(args.text, args.case_name))


def parse_args(args):
    parser = argparse.ArgumentParser(description='Change case style.')
    parser.add_argument('case_name', choices=Case.available_list())
    parser.add_argument('--text')
    return parser.parse_args(args)


def change_case_style(text, case_name):
    in_text = get_text(text)
    out_case = Case.from_string(case_name)

    guesser = CaseGuesser()
    in_case = guesser.guess(in_text)

    splitter = TextSplitter(in_case)
    words = splitter.split(in_text)

    changer = CaseChanger(out_case)
    out_text = changer.change(words)

    return out_text


def get_text(text):
    if text is None:
        text = input().strip()
    return text



class Case(object):
    @staticmethod
    def from_string(case_name):
        cases = [CamelCase, PascalCase, SnakeCase, ConstantCase, KebabCase]

        to_case = {}
        for case in cases:
            to_case.update({flag: case for flag in case.flags()})

        try:
            return to_case[case_name]
        except KeyError:
            raise Exception('invalid case name')

    @staticmethod
    def available_list():
        return CamelCase.flags() + PascalCase.flags() + SnakeCase.flags(
        ) + ConstantCase.flags() + KebabCase.flags()


class SpaceSeparated(object):
    pass


class UnderscoreSeparated(object):
    pass


class HyphenSeparated(object):
    pass


class UppercaseSeparated(object):
    pass


class CapitalCase(SpaceSeparated):
    """ capital case
    Example:
        Case style changer
    """
    pass


class CamelCase(UppercaseSeparated):
    """ camel case
    Example:
        caseStyleChanger
    """
    @staticmethod
    def flags():
        return ['camel', 'camel_case', 'lower_camel_case', 'lcc']


class PascalCase(UppercaseSeparated):
    """ pascal case
    Example:
        CaseStyleChanger
    """
    @staticmethod
    def flags():
        return ['pascal', 'pascal_case', 'upper_camel_case', 'ucc']


class SnakeCase(UnderscoreSeparated):
    """ snake case
    Example:
        case_style_changer
    """
    @staticmethod
    def flags():
        return ['snake', 'snake_case', 'lower_snake_case', 'lsc']


class ConstantCase(UnderscoreSeparated):
    """ constant case
    Example:
        CASE_STYLE_CHANGER
    """
    @staticmethod
    def flags():
        return [
            'constant', 'constant_case', 'screaming', 'screaming_snake_case',
            'upper_snake_case', 'upper_case', 'usc', 'ssc'
        ]


class KebabCase(HyphenSeparated):
    """ kebab case
    Example:
        case-style-changer
    """
    @staticmethod
    def flags():
        return ['kebab', 'kebab_case', 'chain', 'chain_case']



import re

from .case_style import (HyphenSeparated, SpaceSeparated, UnderscoreSeparated,
                         UppercaseSeparated)


class TextSplitter(object):
    def __init__(self, case):
        if case == SpaceSeparated:
            self.split = self.space_separated
        elif case == UnderscoreSeparated:
            self.split = self.underscore_separated
        elif case == HyphenSeparated:
            self.split = self.hyphen_separated
        elif case == UppercaseSeparated:
            self.split = self.uppercase_separated
        else:
            raise Exception('invalid case')

    def space_separated(self, text):
        return text.split(' ')

    def underscore_separated(self, text):
        return text.split('_')

    def hyphen_separated(self, text):
        return text.split('-')

    def uppercase_separated(self, text):
        first_words = [''] + re.findall('[A-Z]', text)
        remaining_words = re.split('[A-Z]', text)

        words = []
        for first_word, remaining_word in zip(first_words, remaining_words):
            word = first_word + remaining_word
            if word != '':
                words.append(word.lower())

        return words



from setuptools import find_packages, setup

import case_style_changer

setup(
    name='case-style-changer',
    version=case_style_changer.__version__,
    description=case_style_changer.__doc__.strip(),
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    author=case_style_changer.__author__,
    author_email='xkumiyu@gmail.com',
    url='https://github.com/xkumiyu/case-style-changer',
    license=case_style_changer.__licence__,
    entry_points={
        'console_scripts': [
            'csc = case_style_changer.cli:main',
            'case_style_changer = case_style_changer.cli:main'
        ]
    },
    packages=find_packages(exclude=('tests', 'docs')),
    setup_requires=['wheel'],
    classifiers=[
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'License :: OSI Approved :: MIT License'
    ],
)
